import PyLibrary.funciones_comunes as fc
import pandas as pd
import os
import re
import logging

def revertir_transformaciones(texto, digitos, secciones):
    # Revertir <> a [[digits]]
    for digito in digitos:
        texto = texto.replace('<>', digito, 1)
    # Revertir <#> a <...> usando las secciones originales
    for seccion in secciones:
        texto = texto.replace('<#>', f'<{seccion}>', 1)
    return texto

def importar_traduccion(tipo_PLC, archivo_maestro, archivo_traduccion, target_lang_code, nivel_afinidad_minimo):
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    master_col = target_lang_code
    translated_col = f"{target_lang_code} Translated"
    affinity_col = f"{target_lang_code} Affinity"

    df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
    df_traduccion = fc.read_dataframe_with_cleanup_retries(archivo_traduccion)

    # Configurar el logger    
    logger = fc.configurar_logger()

    # Iterar sobre las filas del archivo de traducción para actualizar el maestro
    for index, fila in df_traduccion.iterrows():
        clave = fila[df_maestro.columns[0]]
        if clave in df_maestro[df_maestro.columns[0]].values:
            # Comprobar afinidad y valores no nulos/vacíos
            if fila[affinity_col] >= nivel_afinidad_minimo and pd.notnull(fila[translated_col]) and fila[translated_col] != "":
                valor_traducido = fila[translated_col]
                valor_original = df_maestro.loc[df_maestro[df_maestro.columns[0]] == clave, master_col].values[0]
                
                if str(valor_original) != str(valor_traducido):
                    df_maestro.loc[df_maestro[df_maestro.columns[0]] == clave, master_col] = valor_traducido
                    logger.info(f'Fila {index}, Columna {translated_col}: "{valor_original}" actualizado a "{valor_traducido}"')

    # Guardar el archivo maestro actualizado
    fc.save_dataframe_with_retries(df_maestro,output_path=archivo_maestro)
    print(f"Traducciones importadas y archivo maestro actualizado: {archivo_maestro}.")

def run(tipo_PLC, codigo_columna_maestra, seleccion_idioma , nivel_afinidad_minimo):
    archivo_maestro = f".\\data\\1_hmi_master_translates_{tipo_PLC}.xlsx"
    archivo_traduccion = f".\\data\\3_master_export2translate_translated_{tipo_PLC}.xlsx"  
    
    if seleccion_idioma not in fc.IDIOMAS:
        print("Selección inválida.")
    else:
        target_lang, target_lang_code = fc.IDIOMAS[seleccion_idioma]
        importar_traduccion(tipo_PLC, archivo_maestro, archivo_traduccion, target_lang_code, nivel_afinidad_minimo )


if __name__ == "__main__":   
    tipo_PLC = "siemens"
    codigo_columna_maestra = "it-IT"
    nivel_afinidad_minimo = input("Introduce el nivel minimo de afinidad para importar (presiona Enter para usar el valor por defecto 0.5): ")
    nivel_afinidad_minimo = float(nivel_afinidad_minimo) if nivel_afinidad_minimo else 0.5

    fc.mostrar_idiomas()
    seleccion_idioma = int(input("Introduce el número del idioma de destino: "))
    run(tipo_PLC, codigo_columna_maestra, seleccion_idioma, nivel_afinidad_minimo)
